@@ -4,19 +4,20 @@
 
 package Exporter::Proxy;
 
-use v5.8;
+use v5.12;
 use strict;
+use experimental    qw( smartmatch );
 
 use Carp;
 
-use List::Util  qw( first );
-use Symbol      qw( qualify_to_ref );
+use Symbol  qw( qualify_to_ref );
 
 ########################################################################
 # package variables
 ########################################################################
 
-our $VERSION    = '0.06';
+our $VERSION    = '0.07';
+$VERSION = eval $VERSION;
 
 my $disp_list   = 'DISPATCH_OK';
 
@@ -30,34 +31,44 @@ my $disp_list   = 'DISPATCH_OK';
 
 sub import
 {
+    state $stub = sub{};
+
     # discard this package.
     # left on the stack are assignment operators and 
     # exported names.
 
     shift;
 
-    my @exportz = grep { ! /=/ } @_;
-
-    my %argz    = map { /=/ ? ( split /=/ ) : () } @_;
-
-    # maybe carp about extraneous arguments here?
-
-    my $disp    = delete $argz{ dispatch } || '';
-
-    # avoid colliding with '$caller' in the exported subs.
+    # use "$source" avoid colliding with '$caller' in the 
+    # exported subs.
 
     my $source  = caller;
 
+    my @exportz = ();
+
+    my %argz
+    = do
+    {
+        my @argz    = grep /=/, @_;
+        @exportz    = grep { ! ( $_ ~~ @argz ) } @_;
+        
+        map { split /=/ } @argz
+    };
+
+    # maybe carp about extraneous arguments here?
+
+    my $disp    = delete $argz{ dispatch    } || '';
+    my $preproc = delete $argz{ prefilter   } || '';
+
     # if a dispatcher is being used then it must
-    # be exported. in many cases this will be the
+    # be exported. in most cases this will be the
     # only thing exported.
 
     if( $disp )
     {
         my $list    = qualify_to_ref $disp_list, $source;
 
-        first { $disp eq $_ } @exportz
-#        $disp ~~ @exportz 
+        $disp ~~ @exportz 
         or push @exportz, $disp;
 
         unless( $source->can( $disp ) )
@@ -72,7 +83,7 @@ sub import
                 {
                     my $op      = splice @_, 1, 1;
 
-                    first { $op eq $_ } @$sanity
+                    $op ~~ @$sanity
                     or do
                     {
                         local $"    = ' ';
@@ -105,6 +116,25 @@ sub import
         }
     }
 
+    my $pre_handler
+    = do
+    {
+        if( $preproc )
+        {
+            sub
+            {
+                my $handler = $source->can( $preproc )
+                or die "$source cannot '$preproc'\n";
+
+                &$handler
+            }
+        }
+        else
+        {
+            state $stub = sub{}
+        }
+    };
+
     @exportz
     or carp "Oddity: nothing requested for export!";
 
@@ -133,12 +163,17 @@ sub import
         shift;
 
         my $caller  = caller;
+        
+        # allow the caller to pre-process the arguments.
+        # notice this happens *before* ":noexport" is 
+        # processed.
+
+        &$pre_handler;
 
         # empty list => use @exportz.
         # :noexport  => use empty list.
 
-        if( first { ':noexport' eq $_ } @_ )
-#        if( ':noexport' ~~ @_ )
+        if( ':noexport' ~~ @_ )
         {
             @_  = ();
         }
@@ -154,22 +189,21 @@ sub import
         # resolve these at runtime to account for
         # possible autoloading, etc.
 
-        for my $name ( @_ )
+        for( @_ )
         {
-            index $name, ':'
+            index $_, ':'
             or next;
 
-            if( first { $name eq $_ } @exportz )
-#            if( $name ~~ @exportz )
+            if( $_ ~~ @exportz )
             {
-                my $source  = qualify_to_ref $name, $source;
-                my $install = qualify_to_ref $name, $caller;
+                my $source  = qualify_to_ref $_, $source;
+                my $install = qualify_to_ref $_, $caller;
 
                 *$install   = *$source;
             }
             else
             {
-                die "Bogus $source: '$name' not exported";
+                die "Bogus $source: '$_' not exported";
             }
         }
     };
@@ -240,8 +274,7 @@ Exporter::Proxy - Simplified symbol export & proxy dispatch.
     # subroutine, Bar as an array, or foo may 
     # be overloaded with &foo, %foo, @foo, $foo.
 
-    first { $value eq $_ } @Bar
-#    $value ~~ @Bar
+    $value ~~ @Bar
     or croak "Invalid '$value'";
 
     delete $foo{ somekey }
@@ -313,13 +346,9 @@ request.
 =item import
 
 The arguments to this are the symbol names to 
-export with an optional "dispatch=<name>" for
-installing the dispatcher.
-
-The import extracts the exported symbols, adding
-the dispatcher's name to the list if necessary,
-and then installs import, exports, and the 
-dispathcher as necessary.
+export with an optional "dispatch=<name>" or 
+"import=<name>" for installing the dispatcher 
+and extra import handler.
 
 =back
 
@@ -329,6 +358,13 @@ dispathcher as necessary.
 
 =item import
 
+Import handles two optional directives: "dispatch"
+and "filter". The former installs a dispatch and
+adds its name to the exports, the latter allows pre-
+processing the arguments to import without having to
+write an entire import sub to override this one
+(see examples below).
+
 With no arguments the import uses the original
 exports list, pushing all of the symbols into
 the caller's space.
@@ -373,7 +409,7 @@ or
 
 returns an array[ref] copy of the exported names.
 
-=item dispatcher (optional)
+=item dispatch=... (optional)
 
 When exporting a large number of symbols is
 problematic, a dispatcher can be installed 
@@ -383,7 +419,7 @@ and does a goto.
 
 Calls to the dispatcher look like:
 
-    $module->$dispatcher( $name => @name_argz );
+    $module->$dispatch( $name => @name_argz );
 
 The dispatcher splices $name off the stack,
 checks that $module->can( $name ) (or $object
@@ -425,6 +461,80 @@ allows the caller to:
 
     $object->query( lookup => @lookup_argz );
 
+=item prefilter 
+
+Where present this gets first crack at the stack via
+&$subref. This allows the caller to deal with any 
+extra ":foo" arguments without having to write a
+complete import sub. Since Exporter::Proxy's import
+silently ignores anything with a leading colon (other
+than ":noexport") directives can be handled without 
+haviing to munge the stack.
+
+Normal use of this will be the usuall sorts of side-effects
+handled in import subs like reading config files or installing
+variables. This can also deal with any specialized directives 
+by pushing standard lists of exports onto the stack.
+
+    use Exporter::Proxy qw( prefilter=add_groups bim bam foo bar );
+
+    sub bim{}
+    sub bam{}
+
+    sub foo{}
+    sub bar{}
+
+    sub add_groups
+    {
+        state $groupz =
+        {
+            default => [ qw( bim bam ) ],
+            others  => [ qw( foo bar ) ],
+        };
+
+        for my $i ( 0 .. $#_ )
+        {
+            my ( $name ) = $_[$i] =~ m{^ : (\w+) }x
+            or next;
+
+            my $altz    = $groupz->{ $name }
+            or die "Bogus group: '$name' unknown";
+
+            splice @_, $i, 1, @$altz;
+
+            redo
+        }
+    }
+
+The prefilter can consume or munge the stack as necessary. Whatever
+is left will be treated as names for export.
+
+    use Exporter::Proxy qw( prefilter=read_config bim bam foo bar );
+
+    sub read_config
+    {
+        for my $i ( 0 .. $#_ )
+        {
+            my ( $path ) = $_[$i] =~ m{^ : (\w+) }x
+            or next;
+
+            splice @_, $i, 1;
+
+            -e $path    or die "Missing config: '$path'";
+
+            do_something_with $path;
+
+            redo
+        }
+    }
+
+Aside: In later versions of perl
+
+    while( my ( $i, $val ) = each @_ ){ ... }
+
+makes it easier to get $i and the value.
+
+
 =back
 
 A more general use of this is combining a number of 
@@ -438,8 +548,7 @@ becuase the names only propagate up to My::Query.
 My::Query can even use "if" to limit the number
 of services available (e.g., only packages that
 already have an 'IsSafe' method have the modify 
-calls available.
-
+calls available).
 
 =over 4
 
